import datetime

from guet.gateways.gateway import FileGateway, PairSetGateway
from guet.gateways.io import PrintGateway


class PostCommitManager:

    def __init__(self, file_gateway: FileGateway = FileGateway()):
        self._file_gateway = file_gateway

    def manage(self):

        committers = self._file_gateway.get_committers()
        first_committer = committers.pop(0)
        committers.append(first_committer)
        self._file_gateway.set_committers(committers)
        self._file_gateway.set_author_name(committers[0].name)
        self._file_gateway.set_author_email(committers[0].email)


class PreCommitManager:

    def __init__(self, pair_set_gateway: PairSetGateway = PairSetGateway(),
                 print_gateway: PrintGateway = PrintGateway(), exit_method=exit):
        self._print_gateway = print_gateway
        self._pair_set_gateway = pair_set_gateway
        self._exit_method = exit_method

    def manage(self):
        now = round(datetime.datetime.utcnow().timestamp() * 1000)
        twenty_four_hours = 86400000
        twenty_four_hours_ago = now - twenty_four_hours
        if self._pair_set_gateway.get_most_recent_pair_set().set_time < twenty_four_hours_ago:
            self._print_gateway.print("\nYou have not reset pairs in over twenty four hours!\nPlease reset your pairs by using guet set and including your pairs' initials\n")
            self._exit_method(1)
        else:
            self._exit_method(0)



"""
Copyright 2018 Christopher M. Boyer

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from guet.gateways.io import PrintGateway
from .command import Command
from guet.gateways.gateway import *


class SetCommittersCommand(Command):
    _REQUIRED_ARGS_IN_CORRECT_ORDER = ['set']

    def __init__(self,
                 args,
                 user_gateway: UserGateway = UserGateway(),
                 file_gateway: FileGateway = FileGateway(),
                 pair_set_gateway: PairSetGateway = PairSetGateway(),
                 pair_set_committers_gateway: PairSetGatewayCommitterGateway = PairSetGatewayCommitterGateway(),
                 print_gateway: PrintGateway = PrintGateway()):
        super().__init__(args)
        self._pair_set_committers_gateway = pair_set_committers_gateway
        self._pair_set_gateway = pair_set_gateway
        self._user_gateway = user_gateway
        self._file_gateway = file_gateway
        self._print_gateway = print_gateway

    def execute(self):
        committer_initials = self._args[1:]
        committers = []
        pair_set_id = self._pair_set_gateway.add_pair_set(round(datetime.datetime.utcnow().timestamp()*1000))
        pair_set_committer_add = []
        should_set_committers = self._prepare_pair_set_committers(committer_initials, committers, pair_set_committer_add)
        if should_set_committers:
            self._commit_pair_set_committers(committer_initials, committers, pair_set_committer_add, pair_set_id)

    def _commit_pair_set_committers(self, committer_initials, committers, pair_set_committer_add, pair_set_id):
            for pair_set_committer in pair_set_committer_add:
                initials = pair_set_committer
                id = pair_set_id
                self._pair_set_committers_gateway.add_pair_set_committer(initials, id)
            author = self._user_gateway.get_user(committer_initials[0])
            self._file_gateway.set_committers(committers)
            self._file_gateway.set_author_email(author.email)
            self._file_gateway.set_author_name(author.name)

    def _prepare_pair_set_committers(self, committer_initials: list, committers: list, pair_set_committer_add: list):
        should_set_committers = True
        for committer_initial in committer_initials:
            committer = self._user_gateway.get_user(committer_initial)
            if committer is None:
                self._print_gateway.print("No committer exists with initials '{}'".format(committer_initial))
                should_set_committers = False
                break
            committers.append(CommitterInput(name=committer.name, email=committer.email))
            pair_set_committer_add.append(committer_initial)
        return should_set_committers

    def help(self):
        pass

    @classmethod
    def get_short_help_message(cls):
        return 'Set the current committers'

    @classmethod
    def get_list_of_required_arguments_in_correct_order(cls):
        return cls._REQUIRED_ARGS_IN_CORRECT_ORDER



"""
Copyright 2018 Christopher M. Boyer

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class StdoutManager:

    _instance = None

    def __init__(self):
        if self._instance is None:
            StdoutManager._instance = self
        import sys
        self._stdout = sys.__stdout__

    @classmethod
    def get_instance(cls):
        if not cls._instance:
            cls._instance = StdoutManager()
        return cls._instance

    def get_stdout(self):
        return self._stdout

    def set_stdout(self, stdout):
        self._stdout = stdout



from guet.git._create_hook_create_strategy import _HookCreateStrategy


class _CommitMsgStrategy(_HookCreateStrategy):
    def name(self) -> str:
        return 'commit-msg'

    def file_lines(self) -> [str]:
        return [
            "#!/bin/sh\n",
            "FILE_LOCATION=~/.guet/committernames\n",
            'CO_AUTHOR="Co-authored-by:"\n',
            'echo "\\n\\n" >> "$1"\n',
            'while read committer; do\n',
            '	echo "$CO_AUTHOR $committer" >> "$1"\n',
            'done <$FILE_LOCATION\n'
        ]




import unittest
from unittest.mock import Mock

from guet.executor import Executor
from guet.main import main


class TestMain(unittest.TestCase):

    def test_main_calls_executor_execute_with_system_arguments(self):

        import sys
        sys.argv = ['one', 'two']
        expected_args = ['two']

        mock_executor = Executor()
        mock_executor.execute = Mock()

        main(mock_executor)
        mock_executor.execute.assert_called_with(expected_args)




from guet.git._create_hook_create_strategy import _HookCreateStrategy


class _PreCommitStrategy(_HookCreateStrategy):
    def name(self) -> str:
        return 'pre-commit'

    def file_lines(self) -> [str]:
        return [
            '#! /usr/bin/env python3\n',
            'from guet.commit import PreCommitManager\n',
            'cm = PreCommitManager()\n',
            'cm.manage()\n'
        ]




import unittest
from unittest.mock import Mock

from guet.commands.command import Command
from guet.executor import Executor
from guet.factory import CommandFactory


class TestExecutor(unittest.TestCase):

    def test_init_loads_command_factory_by_default(self):

        executor = Executor()
        self.assertIsNotNone(executor._command_factory)

    def test_execute_loads_a_command_from_command_factory_and_executes_it(self):

        command = Command([])
        command.execute = Mock()
        command_factory = CommandFactory()
        command_factory.create = Mock(return_value=command)
        executor = Executor(command_factory)

        args = []
        executor.execute(args)
        command_factory.create.assert_called_with(args)
        command.execute.assert_called()



from enum import Enum
from os import chmod, stat

from guet.git._create_hook_post_commit_strategy import _PostCommitStrategy
from guet.git._create_hook_pre_commit_strategy import _PreCommitStrategy
from guet.git._create_hook_commit_msg_strategy import _CommitMsgStrategy
from guet.git._hook_mode import HookMode


class Hooks(Enum):
    PRE_COMMIT = _PreCommitStrategy
    COMMIT_MSG = _CommitMsgStrategy
    POST_COMMIT = _PostCommitStrategy


def create_hook(hook_folder_path: str, hook: Hooks, create_mode: HookMode):
    hook_strategy = hook.value(hook_folder_path, create_mode)
    final_hook_path = hook_strategy.final_hook_path()
    f = open(final_hook_path, 'w')
    f.writelines(hook_strategy.file_lines())
    f.close()

    st = stat(final_hook_path)
    chmod(final_hook_path, st.st_mode | 0o111)



"""
Copyright 2018 Christopher M. Boyer

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from guet.gateways.io import PrintGateway
from .command import Command
from guet.gateways.gateway import *


class InitDataSourceCommand(Command):
    _REQUIRED_ARGS_IN_CORRECT_ORDER = ['init']

    def __init__(self,
                 args,
                 file_gateway: FileGateway = FileGateway(),
                 print_gateway: PrintGateway = PrintGateway()):
        super().__init__(args, print_gateway)
        self.file_gateway = file_gateway

    def execute(self):
        if len(self._args) != 1:
            self._print_gateway.print('Invalid arguments.\n\n   {}'.format(self.help()))
        else:
            if not self.file_gateway.path_exists():
                self.file_gateway.initialize()
            else:
                self._print_gateway.print('Config folder already exists.')

    def help(self):
        pass

    @classmethod
    def get_short_help_message(cls):
        return 'Initialize guet for use'

    @classmethod
    def get_list_of_required_arguments_in_correct_order(cls):
        return cls._REQUIRED_ARGS_IN_CORRECT_ORDER



"""
Copyright 2018 Christopher M. Boyer

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from guet.gateways.io import PrintGateway, InputGateway
from guet.commands.command import Command
from guet.git.any_hooks_present import any_hooks_present
from guet.git.git_path_from_cwd import git_hook_path_from_cwd
from guet.git.create_hook import create_hook, HookMode, Hooks
from guet.git.git_present_in_cwd import git_present_in_cwd


class StartCommand(Command):
    _REQUIRED_ARGS_IN_CORRECT_ORDER = ['start']

    def __init__(self,
                 args,
                 print_gateway: PrintGateway = PrintGateway(),
                 input_gateway: InputGateway = InputGateway()):
        super().__init__(args, print_gateway)
        self._input_gateway = input_gateway

    def execute(self):
        if git_present_in_cwd():
            hook_path = git_hook_path_from_cwd()
            if not any_hooks_present(hook_path):
                self._create_all_hooks(hook_path, HookMode.NEW_OR_OVERWRITE)
            else:
                hook_mode = None
                self._print_gateway.print('There is already commit hooks in this project. Would you like to overwrite (o), create (c) the file and put it in the hooks folder, or cancel (x)?')
                val = self._input_gateway.input()
                if val == 'o':
                    hook_mode = HookMode.NEW_OR_OVERWRITE
                elif val == 'c':
                    hook_mode = HookMode.CREATE_ALONGSIDE
                else:
                    hook_mode = HookMode.CANCEL
                self._create_all_hooks(hook_path, hook_mode)

        else:
            self._print_gateway.print('Git not initialized in this directory.')

    def _create_all_hooks(self, hook_folder_path: str, hook_mode: HookMode):
        if hook_mode is not HookMode.CANCEL:
            create_hook(hook_folder_path, Hooks.PRE_COMMIT, hook_mode)
            create_hook(hook_folder_path, Hooks.POST_COMMIT, hook_mode)
            create_hook(hook_folder_path, Hooks.COMMIT_MSG, hook_mode)

    def help(self):
        pass

    @classmethod
    def get_short_help_message(cls):
        return 'Start guet usage in the repository at current directory'

    @classmethod
    def get_list_of_required_arguments_in_correct_order(cls):
        return cls._REQUIRED_ARGS_IN_CORRECT_ORDER




import datetime
import sys
import unittest
from unittest.mock import Mock

from guet.commit import PostCommitManager, PreCommitManager
from guet.gateways.gateway import FileGateway, committer_result, PairSetGateway, pair_set_result
from guet.gateways.io import PrintGateway


class PostCommitManagerTest(unittest.TestCase):

    def setUp(self):
        self.mock_file_gateway = FileGateway()
        self.mock_file_gateway.get_committers = Mock()
        self.mock_file_gateway.set_committers = Mock()
        self.mock_file_gateway.set_author_name = Mock()
        self.mock_file_gateway.set_author_email = Mock()

        self.commit_manager = PostCommitManager(self.mock_file_gateway)

    def test_manage_rotates_the_commit_names(self):
        committer1 = committer_result(name='Name', email='email', initials='')
        committer2 = committer_result(name='Name2', email='email', initials='')
        result = [committer1, committer2]
        self.mock_file_gateway.get_committers = Mock(return_value=result)

        self.commit_manager.manage()
        self.mock_file_gateway.set_committers.assert_called_once_with([committer2, committer1])

    def test_manage_sets_the_new_author_name_and_email(self):
        committer1 = committer_result(name='Name', email='email', initials='')
        committer2 = committer_result(name='Name2', email='email', initials='')
        result = [committer1, committer2]
        self.mock_file_gateway.get_committers = Mock(return_value=result)

        self.commit_manager.manage()
        self.mock_file_gateway.set_author_email.assert_called_once_with(committer2.email)
        self.mock_file_gateway.set_author_name.assert_called_once_with(committer2.name)


class PreCommitManagerTest(unittest.TestCase):

    def tearDown(self):
        sys.stdout = self.original_stdout

    def setUp(self):
        self.mock_pair_set_gateway = PairSetGateway()
        self.mock_pair_set_gateway.get_pair_set = Mock()
        self.mock_pair_set_gateway.add_pair_set = Mock()
        self.mock_pair_set_gateway.get_most_recent_pair_set = Mock()

        self.mock_print_gateway = PrintGateway()
        self.mock_print_gateway.print = Mock()

        self.mock_exit_method = Mock()
        self.original_stdout = sys.stdout

    def test_manage_checks_for_most_recent_pair_set_and_exits_1_if_it_is_over_24_hours(self):
        twenty_four_hours = 86400000
        # offset timestamp by 1 so because test might complete in under a second, causing it to fail
        now = round((datetime.datetime.utcnow().timestamp()-1)*1000)

        def _mock_most_recent_pair_set():
            return pair_set_result(id=1, set_time=now - twenty_four_hours)
        self.mock_pair_set_gateway.get_most_recent_pair_set = Mock(side_effect=_mock_most_recent_pair_set)
        subject = PreCommitManager(self.mock_pair_set_gateway, self.mock_print_gateway, self.mock_exit_method)
        subject.manage()
        self.mock_exit_method.assert_called_once_with(1)
        self.mock_print_gateway.print.assert_called_once_with("\nYou have not reset pairs in over twenty four hours!\nPlease reset your pairs by using guet set and including your pairs' initials\n")

    def test_manage_checks_for_most_recent_pair_set_and_exits_0_if_it_is_under_24_hours(self):
        ten_hours = 36000000
        now = round(datetime.datetime.utcnow().timestamp()*1000)

        def _mock_most_recent_pair_set():
            return pair_set_result(id=1, set_time=now - ten_hours)
        self.mock_pair_set_gateway.get_most_recent_pair_set = Mock(side_effect=_mock_most_recent_pair_set)
        subject = PreCommitManager(self.mock_pair_set_gateway, self.mock_print_gateway, self.mock_exit_method)
        subject.manage()
        self.mock_exit_method.assert_called_once_with(0)



from setuptools import setup

import guet

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='guet',
    version=guet.__version__,
    description='Enable contribution tracking when pair programming',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/chiptopher/guet',
    keywords='pair programming',
    entry_points={
        'console_scripts': [
            'guet=guet.main:main'
        ]
    },
    packages=[
        'guet',
        'guet.commands',
        'guet.gateways',
        'guet.git'
    ]
)



from os.path import join

from guet.git._hook_mode import HookMode


class _HookCreateStrategy:

    def __init__(self, hook_folder_path: str, create_mode: HookMode):
        self.create_mode = create_mode
        self.hook_folder_path = hook_folder_path

    def final_hook_path(self):
        return join(self.hook_folder_path, self.create_mode(self.name()))

    def name(self) -> str:
        raise NotImplementedError

    def file_lines(self) -> [str]:
        raise NotImplementedError



import builtins
import sys

from guet.stdout_manager import StdoutManager


class InputGateway:

    def __init__(self, input_method=builtins.input):
        self._input_method = input_method

    def input(self):
        return self._input_method()


class PrintGateway:
    def __init__(self, stdout_manager=StdoutManager.get_instance()):
        self._stdout_manager = stdout_manager

    def print(self, text):
        original_stdout = sys.stdout
        sys.stdout = self._stdout_manager.get_stdout()
        print(text)
        sys.stdout = original_stdout
